@@ -47,16 +47,16 @@
 #' @export
 me <- function(x, sdx, gr = NULL) {
   # use 'term' for consistency with other special terms
-  term <- deparse(substitute(x))
-  sdx <- deparse(substitute(sdx))
+  term <- deparse0(substitute(x))
+  sdx <- deparse0(substitute(sdx))
   gr <- substitute(gr)
   if (!is.null(gr)) {
-    gr <- deparse_combine(gr)
+    gr <- deparse0(gr)
     stopif_illegal_group(gr)
   } else {
     gr <- ""
   }
-  label <- deparse(match.call())
+  label <- deparse0(match.call())
   out <- nlist(term, sdx, gr, label)
   class(out) <- c("me_term", "sp_term")
   out
@@ -130,19 +130,19 @@ me <- function(x, sdx, gr = NULL) {
 #' @export
 mi <- function(x, idx = NA) {
   # use 'term' for consistency with other special terms
-  term <- deparse(substitute(x))
+  term <- deparse0(substitute(x))
   term_vars <- all_vars(term)
   if (!is_equal(term, term_vars)) {
     stop2("'mi' only accepts single untransformed variables.")
   }
-  idx <- deparse(substitute(idx))
+  idx <- deparse0(substitute(idx))
   if (idx != "NA") {
     idx_vars <- all_vars(idx)
     if (!is_equal(idx, idx_vars)) {
       stop2("'mi' only accepts single untransformed variables.")
     }
   }
-  label <- deparse(match.call())
+  label <- deparse0(match.call())
   out <- nlist(term, idx, label)
   class(out) <- c("mi_term", "sp_term")
   out
@@ -203,9 +203,9 @@ mi <- function(x, idx = NA) {
 #' @export
 mo <- function(x, id = NA) {
   # use 'term' for consistency with other special terms
-  term <- deparse(substitute(x))
+  term <- deparse0(substitute(x))
   id <- as_one_character(id, allow_na = TRUE)
-  label <- deparse(match.call())
+  label <- deparse0(match.call())
   out <- nlist(term, id, label)
   class(out) <- c("mo_term", "sp_term")
   out
@@ -359,7 +359,7 @@ tidy_spef <- function(x, data) {
     return(empty_data_frame())
   }
   mm <- sp_model_matrix(form, data, rename = FALSE)
-  out <- data.frame(term = trim_wsp(colnames(mm)), stringsAsFactors = FALSE)
+  out <- data.frame(term = colnames(mm), stringsAsFactors = FALSE)
   out$coef <- rename(out$term)
   calls_cols <- c(paste0("calls_", all_sp_types()), "joint_call")
   list_cols <- c("vars_mi", "idx_mi", "idx2_mi", "ids_mo", "Imo")
@@ -485,7 +485,7 @@ get_sdy <- function(x, data = NULL) {
 get_me_groups <- function(x) {
   uni_me <- get_uni_me(x)
   out <- lapply(uni_me, eval2)
-  out <- ulapply(out, "[[", "gr")
+  out <- ufrom_list(out, "gr")
   out[nzchar(out)]
 }
 
@@ -517,6 +517,8 @@ sp_model_matrix <- function(formula, data, types = all_sp_types(), ...) {
   new_formula <- str2formula(terms_comb)
   attributes(new_formula) <- attributes(formula)
   out <- get_model_matrix(new_formula, data, ...)
+  # fixes issue #1504
+  colnames(out) <- rm_wsp(colnames(out))
   # recover original column names
   colnames(out) <- rename(colnames(out), dummies, terms_replace)
   out
@@ -577,6 +579,7 @@ get_mo_values <- function(term, data) {
   x <- eval2(term$term, data)
   if (is.ordered(x)) {
     # counting starts at zero
+    max_value <- length(levels(x)) - 1
     x <- as.numeric(x) - 1
   } else if (all(is_wholenumber(x))) {
     min_value <- attr(x, "min")
@@ -584,13 +587,16 @@ get_mo_values <- function(term, data) {
       min_value <- min(x)
     }
     x <- x - min_value
+    max_value <- max(x)
   } else {
     stop2(
       "Monotonic predictors must be integers or ordered ",
       "factors. Error occurred for variable '", term$term, "'."
     )
   }
-  as.array(x)
+  x <- as.array(x)
+  attr(x, "max") <- max_value
+  x
 }
 
 # prepare 'sp_term' objects
