@@ -245,9 +245,10 @@ log_lik_student_mv <- function(i, prep) {
 
 log_lik_gaussian_time <- function(i, prep) {
   obs <- with(prep$ac, begin_tg[i]:end_tg[i])
+  Jtime <- prep$ac$Jtime_tg[i, ]
   Y <- as.numeric(prep$data$Y[obs])
   mu <- as.matrix(get_dpar(prep, "mu", i = obs))
-  Sigma <- get_cov_matrix_ac(prep, obs)
+  Sigma <- get_cov_matrix_ac(prep, obs, Jtime = Jtime)
   .log_lik <- function(s) {
     C <- as.matrix(Sigma[s, , ])
     Cinv <- solve(C)
@@ -264,10 +265,11 @@ log_lik_gaussian_time <- function(i, prep) {
 
 log_lik_student_time <- function(i, prep) {
   obs <- with(prep$ac, begin_tg[i]:end_tg[i])
+  Jtime <- prep$ac$Jtime_tg[i, ]
   Y <- as.numeric(prep$data$Y[obs])
   nu <- as.matrix(get_dpar(prep, "nu", i = obs))
   mu <- as.matrix(get_dpar(prep, "mu", i = obs))
-  Sigma <- get_cov_matrix_ac(prep, obs)
+  Sigma <- get_cov_matrix_ac(prep, obs, Jtime = Jtime)
   .log_lik <- function(s) {
     df <- nu[s, ]
     C <- as.matrix(Sigma[s, , ])
@@ -715,6 +717,31 @@ log_lik_hurdle_lognormal <- function(i, prep) {
   log_lik_weight(out, i = i, prep = prep)
 }
 
+log_lik_hurdle_cumulative <- function(i, prep) {
+  mu <- get_dpar(prep, "mu", i = i)
+  hu <- get_dpar(prep, "hu", i = i)
+  disc <- get_dpar(prep, "disc", i = i)
+  thres <- subset_thres(prep, i)
+  nthres <- NCOL(thres)
+  eta <- disc * (thres - mu)
+  y <- prep$data$Y[i]
+  if (y == 0L) {
+    out <- dbinom(1, size = 1, prob = hu, log = TRUE)
+  } else if (y == 1L) {
+    out <- log_cdf(eta[, 1L], prep$family$link) +
+      dbinom(0, size = 1, prob = hu, log = TRUE)
+  } else if (y == nthres + 1L) {
+    out <- log_ccdf(eta[, y - 1L], prep$family$link) +
+      dbinom(0, size = 1, prob = hu, log = TRUE)
+  } else {
+    out <- log_diff_exp(
+      log_cdf(eta[, y], prep$family$link),
+      log_cdf(eta[, y - 1L], prep$family$link)
+    ) + dbinom(0, size = 1, prob = hu, log = TRUE)
+  }
+  log_lik_weight(out, i = i, prep = prep)
+}
+
 log_lik_zero_inflated_poisson <- function(i, prep) {
   zi <- get_dpar(prep, "zi", i)
   lambda <- get_dpar(prep, "mu", i)
@@ -899,7 +926,7 @@ log_lik_acat <- function(i, prep) {
   eta <- disc * (mu - thres)
   y <- prep$data$Y[i]
   # TODO: check if computation can be made more numerically stable
-  if (prep$family$link == "logit") { 
+  if (prep$family$link == "logit") {
     # more efficient computation for logit link
     q <- sapply(1:nthres, function(k) eta[, k])
     p <- cbind(rep(0, nrow(eta)), q[, 1],
@@ -980,14 +1007,22 @@ log_lik_censor <- function(dist, args, i, prep) {
 # @param prep a brmsprep object
 # @return vector of log_lik values
 log_lik_truncate <- function(x, cdf, args, i, prep) {
-  lb <- prep$data$lb[i]
-  ub <- prep$data$ub[i]
-  if (!(is.null(lb) && is.null(ub))) {
-    if (is.null(lb)) lb <- -Inf
-    if (is.null(ub)) ub <- Inf
-    x - log(do_call(cdf, c(ub, args)) - do_call(cdf, c(lb, args)))
+  lb <- prep$data[["lb"]][i]
+  ub <- prep$data[["ub"]][i]
+  if (is.null(lb) && is.null(ub)) {
+    return(x)
   }
-  x
+  if (!is.null(lb)) {
+    log_cdf_lb <- do_call(cdf, c(lb, args, log.p = TRUE))
+  } else {
+    log_cdf_lb <- rep(-Inf, length(x))
+  }
+  if (!is.null(ub)) {
+    log_cdf_ub <- do_call(cdf, c(ub, args, log.p = TRUE))
+  } else {
+    log_cdf_ub <- rep(0, length(x))
+  }
+  x - log_diff_exp(log_cdf_ub, log_cdf_lb)
 }
 
 # weight log_lik values according to defined weights
