@@ -225,7 +225,7 @@ make_stancode <- function(formula, data, family = gaussian(),
     tpar_prior = scode_tpar_prior,
     par_declars = scode_parameters,
     gen_quantities = scode_predictor[["gen_def"]],
-    prior_special = attr(prior, "special"),
+    special_prior = attr(prior, "special"),
     sample_prior = get_sample_prior(prior)
   )
   scode_parameters <- paste0(
@@ -249,7 +249,7 @@ make_stancode <- function(formula, data, family = gaussian(),
       scode_ranef[["tpar_prior_const"]],
       scode_Xme[["tpar_prior_const"]],
       scode_predictor[["tpar_comp"]],
-      scode_predictor[["tpar_reg_prior"]],
+      scode_predictor[["tpar_special_prior"]],
       scode_ranef[["tpar_comp"]],
       scode_Xme[["tpar_comp"]],
       # lprior cannot contain _lupdf functions in transformed parameters
