@@ -6,7 +6,7 @@ prepare_predictions.brmsfit <- function(
   incl_autocor = TRUE, oos = NULL, resp = NULL, ndraws = NULL, draw_ids = NULL,
   nsamples = NULL, subset = NULL, nug = NULL, smooths_only = FALSE,
   offset = TRUE, newdata2 = NULL, new_objects = NULL, point_estimate = NULL,
-  ...
+  ndraws_point_estimate = 1, ...
 ) {
 
   x <- restructure(x)
@@ -29,7 +29,7 @@ prepare_predictions.brmsfit <- function(
   draw_ids <- validate_draw_ids(x, draw_ids, ndraws)
   draws <- as_draws_matrix(x)
   draws <- suppressMessages(subset_draws(draws, draw = draw_ids))
-  draws <- point_draws(draws, point_estimate)
+  draws <- point_draws(draws, point_estimate, ndraws_point_estimate)
 
   new_formula <- update_re_terms(x$formula, re_formula)
   bterms <- brmsterms(new_formula)
@@ -55,12 +55,13 @@ prepare_predictions.brmsfit <- function(
   )
 }
 
+#' @export
 prepare_predictions.mvbrmsterms <- function(x, draws, sdata, resp = NULL, ...) {
   resp <- validate_resp(resp, x$responses)
   if (length(resp) > 1) {
     if (has_subset(x)) {
       stop2("Argument 'resp' must be a single variable name ",
-            "for models using addition argument 'draw_ids'.")
+            "for models using addition argument 'subset'.")
     }
     out <- list(ndraws = nrow(draws), nobs = sdata$N)
     out$resps <- named_list(resp)
@@ -101,7 +102,14 @@ prepare_predictions.brmsterms <- function(x, draws, sdata, data, ...) {
   resp <- usc(combine_prefix(x))
   out <- nlist(ndraws, nobs, resp = x$resp)
   out$family <- prepare_family(x)
+
   out$old_order <- attr(sdata, "old_order")
+  if (has_subset(x) && !is.null(out$old_order)) {
+    # old_order has length equal to the full number of observations
+    # which is inappropriate for subsetted responses (#1483)
+    out$old_order <- as.numeric(factor(out$old_order[attr(data, "subset")]))
+  }
+
   valid_dpars <- valid_dpars(x)
   out$dpars <- named_list(valid_dpars)
   for (dp in valid_dpars) {
@@ -198,10 +206,13 @@ prepare_predictions.btnl <- function(x, draws, sdata, ...) {
   class(out) <- "bprepnl"
   p <- usc(combine_prefix(x))
   covars <- all.vars(x$covars)
-  dim <- c(out$ndraws, out$nobs)
   for (i in seq_along(covars)) {
     cvalues <- sdata[[paste0("C", p, "_", i)]]
-    out$C[[covars[i]]] <- data2draws(cvalues, dim = dim)
+    cdim <- c(out$ndraws, out$nobs)
+    if (is.matrix(cvalues)) {
+      c(cdim) <- dim(cvalues)[2]
+    }
+    out$C[[covars[i]]] <- data2draws(cvalues, dim = cdim)
   }
   out
 }
@@ -621,7 +632,7 @@ prepare_predictions_re <- function(bterms, sdata, prep_ranef = list(),
   }
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
-  ranef_px <- lapply(prep_ranef, "[[", "ranef")
+  ranef_px <- from_list(prep_ranef, "ranef")
   ranef_px <- do_call(rbind, ranef_px)
   ranef_px <- subset2(ranef_px, ls = px)
   if (!NROW(ranef_px)) {
@@ -736,6 +747,11 @@ prepare_predictions_ac <- function(bterms, draws, sdata, oos = NULL,
     cosy_regex <- paste0("^cosy", p, "$")
     out$cosy <- prepare_draws(draws, cosy_regex, regex = TRUE)
   }
+  if (has_ac_class(acef, "unstr")) {
+    cortime_regex <- paste0("^cortime", p, "__")
+    out$cortime <- prepare_draws(draws, cortime_regex, regex = TRUE)
+    out$Jtime_tg <- sdata[[paste0("Jtime_tg", p)]]
+  }
   if (use_ac_cov_time(acef)) {
     # prepare predictions for the covariance structures of time-series models
     out$begin_tg <- sdata[[paste0("begin_tg", p)]]
@@ -757,8 +773,9 @@ prepare_predictions_ac <- function(bterms, draws, sdata, oos = NULL,
       out$sderr <- prepare_draws(draws, sderr_regex, regex = TRUE)
       for (i in seq_len(out$N_tg)) {
         obs <- with(out, begin_tg[i]:end_tg[i])
+        Jtime <- out$Jtime_tg[i, ]
+        cov <- get_cov_matrix_ac(list(ac = out), obs, Jtime = Jtime, latent = TRUE)
         zeros <- rep(0, length(obs))
-        cov <- get_cov_matrix_ac(list(ac = out), obs, latent = TRUE)
         .err <- function(s) rmulti_normal(1, zeros, Sigma = cov[s, , ])
         out$err[, obs] <- rblapply(seq_rows(draws), .err)
       }
@@ -1091,12 +1108,17 @@ prepare_draws <- function(x, variable, ...) {
 # currently used primarily for 'loo_subsample'
 # @param draws matrix of posterior draws
 # @param point_estimate optional name of the point estimate to be computed
+# @param ndraws_point_estimate number of repetitions of the point estimate's
+#   value in the form of pseudo draws
 # @return a draws_matrix with one row
-point_draws <- function(draws, point_estimate = NULL) {
+point_draws <- function(draws, point_estimate = NULL,
+                        ndraws_point_estimate = 1) {
   if (is.null(point_estimate)) {
     return(draws)
   }
   point_estimate <- match.arg(point_estimate, c("mean", "median"))
+  ndraws_point_estimate <- as_one_integer(ndraws_point_estimate)
+  stopifnot(ndraws_point_estimate > 0)
   variables <- colnames(draws)
   if (point_estimate == "mean") {
     draws <- matrixStats::colMeans2(draws)
@@ -1104,6 +1126,10 @@ point_draws <- function(draws, point_estimate = NULL) {
     draws <- matrixStats::colMedians(draws)
   }
   draws <- t(draws)
+  draws <- matrix(
+    draws, nrow = ndraws_point_estimate,
+    ncol = ncol(draws), byrow = TRUE
+  )
   colnames(draws) <- variables
   as_draws_matrix(draws)
 }
@@ -1194,6 +1220,9 @@ is.bprepnl <- function(x) {
 #'   be set to \code{"mean"} or \code{"median"}. This argument is primarily
 #'   implemented to ensure compatibility with the \code{\link{loo_subsample}}
 #'   method.
+#' @param ndraws_point_estimate Only used if \code{point_estimate} is not
+#'   \code{NULL}. How often shall the point estimate's value be repeated?
+#'   Defaults to \code{1}.
 #' @param ... Further arguments passed to \code{\link{validate_newdata}}.
 #'
 #' @return An object of class \code{'brmsprep'} or \code{'mvbrmsprep'},
