@@ -19,10 +19,11 @@ stan_predictor.btl <- function(x, ...) {
     stan_gp(x, ...),
     stan_ac(x, ...),
     stan_offset(x, ...),
-    stan_bhaz(x, ...),
-    stan_special_prior_global(x, ...)
+    stan_bhaz(x, ...)
   )
-  stan_eta_combine(out, bterms = x, ...)
+  out <- stan_special_prior(x, out = out, ...)
+  out <- stan_eta_combine(x, out = out, ...)
+  out
 }
 
 # prepare Stan code for non-linear terms
@@ -125,7 +126,7 @@ stan_predictor.mvbrmsterms <- function(x, prior, threads, normalize, ...) {
   resp_type <- out[[1]]$resp_type
   out <- collapse_lists(ls = out)
   out$resp_type <- "vector"
-  adforms <- lapply(x$terms, "[[", "adforms")
+  adforms <- from_list(x$terms, "adforms")
   adnames <- unique(ulapply(adforms, names))
   adallowed <- c("se", "weights", "mi")
   if (!all(adnames %in% adallowed))  {
@@ -139,7 +140,7 @@ stan_predictor.mvbrmsterms <- function(x, prior, threads, normalize, ...) {
   nresp <- length(resp)
   str_add(out$model_def) <- glue(
     "  // multivariate predictor array\n",
-    "  vector[nresp] Mu[N];\n"
+    "  array[N] vector[nresp] Mu;\n"
   )
   str_add(out$model_comp_mvjoin) <- glue(
     "    Mu[n] = {stan_vector(glue('mu_{resp}[n]'))};\n"
@@ -150,7 +151,7 @@ stan_predictor.mvbrmsterms <- function(x, prior, threads, normalize, ...) {
   )
   str_add(out$pll_args) <- glue(", data int nresp")
   str_add(out$tdata_def) <- glue(
-    "  vector[nresp] Y[N];  // response array\n"
+    "  array[N] vector[nresp] Y;  // response array\n"
   )
   str_add(out$tdata_comp) <- glue(
     "  for (n in 1:N) {{\n",
@@ -165,7 +166,7 @@ stan_predictor.mvbrmsterms <- function(x, prior, threads, normalize, ...) {
     )
     str_add(out$pll_args) <- glue(", data vector weights")
   }
-  miforms <- rmNULL(lapply(adforms, "[[", "mi"))
+  miforms <- rmNULL(from_list(adforms, "mi"))
   if (length(miforms)) {
     str_add(out$model_no_pll_def) <- "  vector[nresp] Yl[N] = Y;\n"
     str_add(out$pll_args) <- ", vector[] Yl"
@@ -192,14 +193,14 @@ stan_predictor.mvbrmsterms <- function(x, prior, threads, normalize, ...) {
   }
   sigma <- ulapply(x$terms, stan_sigma_transform, threads = threads)
   if (any(grepl(stan_nn_regex(), sigma))) {
-    str_add(out$model_def) <- "  vector[nresp] sigma[N];\n"
+    str_add(out$model_def) <- "  array[N] vector[nresp] sigma;\n"
     str_add(out$model_comp_mvjoin) <- glue(
       "    sigma[n] = {stan_vector(sigma)};\n"
     )
     if (family == "gaussian") {
       str_add(out$model_def) <- glue(
         "  // cholesky factor of residual covariance matrix\n",
-        "  matrix[nresp, nresp] LSigma[N];\n"
+        "  array[N] matrix[nresp, nresp] LSigma;\n"
       )
       str_add(out$model_comp_mvjoin) <- glue(
         "    LSigma[n] = diag_pre_multiply(sigma[n], Lrescor);\n"
@@ -207,7 +208,7 @@ stan_predictor.mvbrmsterms <- function(x, prior, threads, normalize, ...) {
     } else if (family == "student") {
       str_add(out$model_def) <- glue(
         "  // residual covariance matrix\n",
-        "  matrix[nresp, nresp] Sigma[N];\n"
+        "  array[N] matrix[nresp, nresp] Sigma;\n"
       )
       str_add(out$model_comp_mvjoin) <- glue(
         "    Sigma[n] = multiply_lower_tri_self_transpose(",
@@ -282,6 +283,8 @@ stan_fe <- function(bterms, data, prior, stanvars, threads, primitive,
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
   resp <- usc(px$resp)
+  lpdf <- stan_lpdf_name(normalize)
+
   if (length(fixef)) {
     str_add(out$data) <- glue(
       "  int<lower=1> K{p};",
@@ -311,47 +314,33 @@ stan_fe <- function(bterms, data, prior, stanvars, threads, primitive,
     }
     # prepare population-level coefficients
     b_type <- glue("vector[K{ct}{p}]")
-    has_special_b_prior <- stan_has_special_b_prior(bterms, prior)
-    assign_b_tpar <- stan_assign_b_tpar(bterms, prior)
+    has_special_prior <- has_special_prior(prior, bterms, class = "b")
     if (decomp == "none") {
-      b_suffix <- ""
-      b_comment <- "population-level effects"
-      if (has_special_b_prior) {
-        stopif_prior_bound(prior, class = "b", ls = px)
-        b_def <- glue("  {b_type} b{p};  // {b_comment}\n")
-        if (assign_b_tpar) {
-          # only some special priors assign b in transformed parameters
-          str_add(out$tpar_def) <- b_def
-        } else {
-          str_add(out$par) <- b_def
-        }
-        str_add(out$pll_args) <- glue(", vector b{p}")
+      if (has_special_prior) {
+        str_add_list(out) <- stan_prior_non_centered(
+          suffix = p, suffix_K = ct, normalize = normalize
+        )
       } else {
         str_add_list(out) <- stan_prior(
           prior, class = "b", coef = fixef, type = b_type,
           px = px, suffix = p, header_type = "vector",
-          comment = b_comment, normalize = normalize
+          comment = "regression coefficients", normalize = normalize
         )
       }
     } else {
       stopifnot(decomp == "QR")
       stopif_prior_bound(prior, class = "b", ls = px)
-      b_suffix <- "Q"
-      b_comment <- "regression coefficients at QR scale"
-      if (has_special_b_prior) {
-        bQ_def <- glue("  {b_type} bQ{p};  // {b_comment}\n")
-        if (assign_b_tpar) {
-          # only some special priors assign b in transformed parameters
-          str_add(out$tpar_def) <- bQ_def
-        } else {
-          str_add(out$par) <- bQ_def
-        }
-        str_add(out$pll_args) <- glue(", vector bQ{p}")
+      if (has_special_prior) {
+        str_add_list(out) <- stan_prior_non_centered(
+          suffix = p, suffix_class = "Q", suffix_K = ct,
+          normalize = normalize
+        )
       } else {
         str_add_list(out) <- stan_prior(
           prior, class = "b", coef = fixef, type = b_type,
           px = px, suffix = glue("Q{p}"), header_type = "vector",
-          comment = b_comment, normalize = normalize
+          comment = "regression coefficients on QR scale",
+          normalize = normalize
         )
       }
       str_add(out$gen_def) <- glue(
@@ -359,11 +348,6 @@ stan_fe <- function(bterms, data, prior, stanvars, threads, primitive,
         "  vector[K{ct}{p}] b{p} = XR{p}_inv * bQ{p};\n"
       )
     }
-    str_add_list(out) <- stan_special_prior_local(
-      prior, class = "b", ncoef = length(fixef),
-      px = px, center_X = center_X, suffix = b_suffix,
-      normalize = normalize
-    )
   }
 
   order_intercepts <- order_intercepts(bterms)
@@ -378,11 +362,13 @@ stan_fe <- function(bterms, data, prior, stanvars, threads, primitive,
     # centering the design matrix improves convergence
     sub_X_means <- ""
     if (length(fixef)) {
+      str_add(out$data) <- glue(
+        "  int<lower=1> Kc{p};",
+        "  // number of population-level effects after centering\n"
+      )
       sub_X_means <- glue(" - dot_product(means_X{p}, b{p})")
       if (is_ordinal(family)) {
-        # the intercept was already removed during the data preparation
         str_add(out$tdata_def) <- glue(
-          "  int Kc{p} = K{p};\n",
           "  matrix[N{resp}, Kc{p}] Xc{p};",
           "  // centered version of X{p}\n",
           "  vector[Kc{p}] means_X{p};",
@@ -396,7 +382,6 @@ stan_fe <- function(bterms, data, prior, stanvars, threads, primitive,
         )
       } else {
         str_add(out$tdata_def) <- glue(
-          "  int Kc{p} = K{p} - 1;\n",
           "  matrix[N{resp}, Kc{p}] Xc{p};",
           "  // centered version of X{p} without an intercept\n",
           "  vector[Kc{p}] means_X{p};",
@@ -519,9 +504,9 @@ stan_re <- function(ranef, prior, normalize, ...) {
   if (r$gtype[1] == "mm") {
     for (res in uresp) {
       str_add(out$data) <- cglue(
-        "  int<lower=1> J_{id}{res}_{ng}[N{res}];",
+        "  array[N{res}] int<lower=1> J_{id}{res}_{ng};",
         "  // grouping indicator per observation\n",
-        "  real W_{id}{res}_{ng}[N{res}];",
+        "  array[N{res}] real W_{id}{res}_{ng};",
         "  // multi-membership weights\n"
       )
       str_add(out$pll_args) <- cglue(
@@ -530,7 +515,7 @@ stan_re <- function(ranef, prior, normalize, ...) {
     }
   } else {
     str_add(out$data) <- cglue(
-      "  int<lower=1> J_{id}{uresp}[N{uresp}];",
+      "  array[N{uresp}] int<lower=1> J_{id}{uresp};",
       "  // grouping indicator per observation\n"
     )
     str_add(out$pll_args) <- cglue(
@@ -540,7 +525,7 @@ stan_re <- function(ranef, prior, normalize, ...) {
   if (has_by) {
     str_add(out$data) <- glue(
       "  int<lower=1> Nby_{id};  // number of by-factor levels\n",
-      "  int<lower=1> Jby_{id}[N_{id}];",
+      "  array[N_{id}] int<lower=1> Jby_{id};",
       "  // by-factor indicator per observation\n"
     )
   }
@@ -572,8 +557,14 @@ stan_re <- function(ranef, prior, normalize, ...) {
       )
     }
   }
+
   # define standard deviation parameters
+  has_special_prior <- has_special_prior(prior, px, class = "sd")
   if (has_by) {
+    if (has_special_prior) {
+      stop2("Special priors on class 'sd' are not yet compatible ",
+            "with the 'by' argument.")
+    }
     str_add_list(out) <- stan_prior(
       prior, class = "sd", group = r$group[1], coef = r$coef,
       type = glue("matrix[M_{id}, Nby_{id}]"),
@@ -583,13 +574,25 @@ stan_re <- function(ranef, prior, normalize, ...) {
       normalize = normalize
     )
   } else {
-    str_add_list(out) <- stan_prior(
-      prior, class = "sd", group = r$group[1], coef = r$coef,
-      type = glue("vector[M_{id}]"), suffix = glue("_{id}"), px = px,
-      comment = "group-level standard deviations",
-      normalize = normalize
-    )
+    if (has_special_prior) {
+      if (stan_has_multiple_base_priors(px)) {
+        stop2("Special priors on class 'sd' are not yet compatible with ",
+              "group-level coefficients correlated across formulas.")
+      }
+      str_add(out$tpar_def) <- glue(
+        "  vector<lower=0>[M_{id}] sd_{id};  // group-level standard deviations\n"
+      )
+    } else {
+      str_add_list(out) <- stan_prior(
+        prior, class = "sd", group = r$group[1], coef = r$coef,
+        type = glue("vector[M_{id}]"), suffix = glue("_{id}"), px = px,
+        comment = "group-level standard deviations",
+        normalize = normalize
+      )
+    }
   }
+
+  # define group-level coefficients
   dfm <- ""
   tr <- get_dist_groups(r, "student")
   if (nrow(r) > 1L && r$cor[1]) {
@@ -685,7 +688,7 @@ stan_re <- function(ranef, prior, normalize, ...) {
   } else {
     # single or uncorrelated group-level effects
     str_add(out$par) <- glue(
-      "  vector[N_{id}] z_{id}[M_{id}];",
+      "  array[M_{id}] vector[N_{id}] z_{id};",
       "  // standardized group-level effects\n"
     )
     str_add(out$model_prior) <- cglue(
@@ -741,12 +744,19 @@ stan_sm <- function(bterms, data, prior, threads, normalize, ...) {
       "  // design matrix for the linear effects\n"
     )
     str_add(out$pll_args) <- glue(", data matrix Xs{p}")
-    str_add_list(out) <- stan_prior(
-      prior, class = "b", coef = Xs_names,
-      type = glue("vector[Ks{p}]"), suffix = glue("s{p}"),
-      header_type = "vector", px = px,
-      comment = "spline coefficients", normalize = normalize
-    )
+    if (has_special_prior(prior, px, class = "b")) {
+      str_add_list(out) <- stan_prior_non_centered(
+        suffix = glue("s{p}"), normalize = normalize
+      )
+    } else {
+      str_add_list(out) <- stan_prior(
+        prior, class = "b", coef = Xs_names,
+        type = glue("vector[Ks{p}]"), suffix = glue("s{p}"),
+        header_type = "vector", px = px,
+        comment = "unpenalized spline coefficients",
+        normalize = normalize
+      )
+    }
     str_add(out$eta) <- glue(" + Xs{p}{slice} * bs{p}")
   }
   for (i in seq_rows(smef)) {
@@ -756,9 +766,9 @@ stan_sm <- function(bterms, data, prior, threads, normalize, ...) {
     pi <- glue("{p}_{i}")
     nb <- seq_len(smef$nbases[[i]])
     str_add(out$data) <- glue(
-      "  // data for spline {smef$byterm[i]}\n",
+      "  // data for spline {i}\n",
       "  int nb{pi};  // number of bases\n",
-      "  int knots{pi}[nb{pi}];  // number of knots\n"
+      "  array[nb{pi}] int knots{pi};  // number of knots\n"
     )
     str_add(out$data) <- "  // basis function matrices\n"
     str_add(out$data) <- cglue(
@@ -766,28 +776,35 @@ stan_sm <- function(bterms, data, prior, threads, normalize, ...) {
     )
     str_add(out$pll_args) <- cglue(", data matrix Zs{pi}_{nb}")
     str_add(out$par) <- glue(
-      "  // parameters for spline {smef$byterm[i]}\n"
+      "  // parameters for spline {i}\n"
     )
     str_add(out$par) <- cglue(
-      "  // standarized spline coefficients\n",
+      "  // standardized penalized spline coefficients\n",
       "  vector[knots{pi}[{nb}]] zs{pi}_{nb};\n"
     )
-    for (j in nb) {
+    if (has_special_prior(prior, px, class = "sds")) {
+      str_add(out$tpar_def) <- glue(
+        "  // SDs of penalized spline coefficients\n",
+        "  vector<lower=0>[nb{pi}] sds{pi};\n"
+      )
+      str_add(out$prior_global_scales) <- glue(" sds{pi}")
+      str_add(out$prior_global_lengths) <- glue(" nb{pi}")
+    } else {
       str_add_list(out) <- stan_prior(
-        prior, class = "sds", coef = smef$term[i],
-        suffix = glue("{pi}_{j}"), px = px,
-        comment = "standard deviations of spline coefficients",
+        prior, class = "sds", coef = smef$term[i], suffix = pi, px = px,
+        type = glue("vector[nb{pi}]"), coef_type = glue("vector[nb{pi}]"),
+        comment = "SDs of penalized spline coefficients",
         normalize = normalize
       )
     }
     # separate definition from computation to support fixed parameters
     str_add(out$tpar_def) <- cglue(
-      "  // actual spline coefficients\n",
+      "  // penalized spline coefficients\n",
       "  vector[knots{pi}[{nb}]] s{pi}_{nb};\n"
     )
-    str_add(out$tpar_comp) <- cglue(
-      "  // compute actual spline coefficients\n",
-      "  s{pi}_{nb} = sds{pi}_{nb} * zs{pi}_{nb};\n"
+    str_add(out$tpar_special_prior) <- cglue(
+      "  // compute penalized spline coefficients\n",
+      "  s{pi}_{nb} = sds{pi}[{nb}] * zs{pi}_{nb};\n"
     )
     str_add(out$pll_args) <- cglue(", vector s{pi}_{nb}")
     str_add(out$model_prior) <- cglue(
@@ -873,10 +890,13 @@ stan_sp <- function(bterms, data, prior, stanvars, ranef, meef, threads,
                     normalize, ...) {
   out <- list()
   spef <- tidy_spef(bterms, data)
-  if (!nrow(spef)) return(out)
+  if (!nrow(spef)) {
+    return(out)
+  }
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
   resp <- usc(px$resp)
+  lpdf <- stan_lpdf_name(normalize)
   n <- stan_nn(threads)
   ranef <- subset2(ranef, type = "sp", ls = px)
   spef_coef <- rename(spef$term)
@@ -936,7 +956,7 @@ stan_sp <- function(bterms, data, prior, stanvars, ranef, meef, threads,
   if (any(which_Imo)) {
     str_add(out$data) <- glue(
       "  int<lower=1> Imo{p};  // number of monotonic variables\n",
-      "  int<lower=1> Jmo{p}[Imo{p}];  // length of simplexes\n"
+      "  array[Imo{p}] int<lower=1> Jmo{p};  // length of simplexes\n"
     )
     ids <- unlist(spef$ids_mo)
     lpdf <- stan_lpdf_name(normalize)
@@ -947,7 +967,7 @@ stan_sp <- function(bterms, data, prior, stanvars, ranef, meef, threads,
         # index of first ID appearance
         j_id <- match(id, ids)
         str_add(out$data) <- glue(
-          "  int Xmo{p}_{j}[N{resp}];  // monotonic variable\n"
+          "  array[N{resp}] int Xmo{p}_{j};  // monotonic variable\n"
         )
         str_add(out$pll_args) <- glue(
           ", int[] Xmo{p}_{j}, vector simo{p}_{j}"
@@ -979,25 +999,17 @@ stan_sp <- function(bterms, data, prior, stanvars, ranef, meef, threads,
   for (j in seq_rows(uni_mi)) {
     idxl <- glue("idxl{p}_{uni_mi$var[j]}_{uni_mi$idx2[j]}")
     str_add(out$data) <- glue(
-      "  int {idxl}[N{resp}];  // matching indices\n"
+      "  array[N{resp}] int {idxl};  // matching indices\n"
     )
     str_add(out$pll_args) <- glue(", data int[] {idxl}")
   }
 
   # prepare special effects coefficients
-  if (stan_has_special_b_prior(bterms, prior)) {
+  if (has_special_prior(prior, bterms, class = "b")) {
     stopif_prior_bound(prior, class = "b", ls = px)
-    bsp_def <- glue(
-      "  // special effects coefficients\n",
-      "  vector[Ksp{p}] bsp{p};\n"
+    str_add_list(out) <- stan_prior_non_centered(
+      suffix = glue("sp{p}"), normalize = normalize
     )
-    if (stan_assign_b_tpar(bterms, prior)) {
-      # only some special priors assign b in transformed parameters
-      str_add(out$tpar_def) <- bsp_def
-    } else {
-      str_add(out$par) <- bsp_def
-    }
-    str_add(out$pll_args) <- glue(", vector bsp{p}")
   } else {
     str_add_list(out) <- stan_prior(
       prior, class = "b", coef = spef$coef,
@@ -1007,11 +1019,6 @@ stan_sp <- function(bterms, data, prior, stanvars, ranef, meef, threads,
       normalize = normalize
     )
   }
-  stan_special_priors <- stan_special_prior_local(
-    prior, class = "bsp", ncoef = nrow(spef),
-    px = px, center_X = FALSE, normalize = normalize
-  )
-  out <- collapse_lists(out, stan_special_priors)
   out
 }
 
@@ -1049,12 +1056,20 @@ stan_gp <- function(bterms, data, prior, threads, normalize, ...) {
         "  int<lower=1> NBgp{pi};\n"
       )
     }
-    str_add_list(out) <- stan_prior(
-      prior, class = "sdgp", coef = sfx1,
-      type = glue("vector[Kgp{pi}]"), px = px, suffix = pi,
-      comment = "GP standard deviation parameters",
-      normalize = normalize
-    )
+    if (has_special_prior(prior, px, class = "sdgp")) {
+      str_add(out$tpar_def) <- glue(
+        "  vector<lower=0>[Kgp{pi}] sdgp{pi};  // GP standard deviation parameters\n"
+      )
+      str_add(out$prior_global_scales) <- glue(" sdgp{pi}")
+      str_add(out$prior_global_lengths) <- glue(" Kgp{pi}")
+    } else {
+      str_add_list(out) <- stan_prior(
+        prior, class = "sdgp", coef = sfx1, px = px, suffix = pi,
+        type = glue("vector[Kgp{pi}]"), coef_type = glue("vector[Kgp{pi}]"),
+        comment = "GP standard deviation parameters",
+        normalize = normalize
+      )
+    }
     if (gpef$iso[i]) {
       lscale_type <- "vector[1]"
       lscale_dim <- glue("[Kgp{pi}]")
@@ -1071,12 +1086,12 @@ stan_gp <- function(bterms, data, prior, threads, normalize, ...) {
       Igp <- glue("Igp{pi}_{J}")
       str_add(out$data) <- glue(
         "  // number of observations relevant for a certain sub-GP\n",
-        "  int<lower=1> {Ngp}[Kgp{pi}];\n"
+        "  array[Kgp{pi}] int<lower=1> {Ngp};\n"
       )
       str_add(out$data) <-
         "  // indices and contrasts of sub-GPs per observation\n"
       str_add(out$data) <- cglue(
-        "  int<lower=1> {Igp}[{Ngp}[{J}]];\n",
+        "  array[{Ngp}[{J}]] int<lower=1> {Igp};\n",
         "  vector[{Ngp}[{J}]] Cgp{pi}_{J};\n"
       )
       str_add(out$pll_args) <- cglue(
@@ -1090,11 +1105,11 @@ stan_gp <- function(bterms, data, prior, threads, normalize, ...) {
       if (gr) {
         str_add(out$data) <- glue(
           "  // number of latent GP groups\n",
-          "  int<lower=1> Nsubgp{pi}[Kgp{pi}];\n"
+          "  array[Kgp{pi}] int<lower=1> Nsubgp{pi};\n"
         )
         str_add(out$data) <- cglue(
           "  // indices of latent GP groups per observation\n",
-          "  int<lower=1> Jgp{pi}_{J}[{Ngp}[{J}]];\n"
+          "  array[{Ngp}[{J}]] int<lower=1> Jgp{pi}_{J};\n"
         )
         str_add(out$pll_args) <- cglue(", data int[] Jgp{pi}_{J}")
       }
@@ -1103,7 +1118,7 @@ stan_gp <- function(bterms, data, prior, threads, normalize, ...) {
           "  // approximate GP basis matrices and eigenvalues\n"
         str_add(out$data) <- cglue(
           "  matrix[{Nsubgp}[{J}], NBgp{pi}] Xgp{pi}_{J};\n",
-          "  vector[Dgp{pi}] slambda{pi}_{J}[NBgp{pi}];\n"
+          "  array[NBgp{pi}] vector[Dgp{pi}] slambda{pi}_{J};\n"
         )
         str_add(out$par) <- "  // latent variables of the GP\n"
         str_add(out$par) <- cglue(
@@ -1119,7 +1134,7 @@ stan_gp <- function(bterms, data, prior, threads, normalize, ...) {
         # exact GPs
         str_add(out$data) <- "  // covariates of the GP\n"
         str_add(out$data) <- cglue(
-          "  vector[Dgp{pi}] Xgp{pi}_{J}[{Nsubgp}[{J}]];\n"
+          "  array[{Nsubgp}[{J}]] vector[Dgp{pi}] Xgp{pi}_{J};\n"
         )
         str_add(out$par) <- "  // latent variables of the GP\n"
         str_add(out$par) <- cglue(
@@ -1168,7 +1183,7 @@ stan_gp <- function(bterms, data, prior, threads, normalize, ...) {
           "  // number of latent GP groups\n",
           "  int<lower=1> {Nsubgp};\n",
           "  // indices of latent GP groups per observation\n",
-          "  int<lower=1> Jgp{pi}[N{resp}];\n"
+          "  array[N{resp}] int<lower=1> Jgp{pi};\n"
         )
         str_add(out$pll_args) <- glue(", data int[] Jgp{pi}")
       }
@@ -1186,7 +1201,7 @@ stan_gp <- function(bterms, data, prior, threads, normalize, ...) {
           "  // approximate GP basis matrices\n",
           "  matrix[{Nsubgp}, NBgp{pi}] Xgp{pi};\n",
           "  // approximate GP eigenvalues\n",
-          "  vector[Dgp{pi}] slambda{pi}[NBgp{pi}];\n"
+          "  array[NBgp{pi}] vector[Dgp{pi}] slambda{pi};\n"
         )
         str_add(out$par) <- glue(
           "  vector[NBgp{pi}] zgp{pi};  // latent variables of the GP\n"
@@ -1214,7 +1229,7 @@ stan_gp <- function(bterms, data, prior, threads, normalize, ...) {
       } else {
         # exact GPs
         str_add(out$data) <- glue(
-          "  vector[Dgp{pi}] Xgp{pi}[{Nsubgp}];  // covariates of the GP\n"
+          "  array[{Nsubgp}] vector[Dgp{pi}] Xgp{pi};  // covariates of the GP\n"
         )
         str_add(out$par) <- glue(
           "  vector[{Nsubgp}] zgp{pi};  // latent variables of the GP\n"
@@ -1260,10 +1275,18 @@ stan_ac <- function(bterms, data, prior, threads, normalize, ...) {
     str_add(out$par) <- glue(
       "  vector[N{resp}] zerr{p};  // unscaled residuals\n"
     )
-    str_add_list(out) <- stan_prior(
-      prior, class = "sderr", px = px, suffix = p,
-      comment = "SD of residuals", normalize = normalize
-    )
+    if (has_special_prior(prior, px, class = "sderr")) {
+      str_add(out$tpar_def) <- glue(
+        "  real<lower=0> sderr{p};  // SD of residuals\n"
+      )
+      str_add(out$prior_global_scales) <- glue(" sderr{p}")
+      str_add(out$prior_global_lengths) <- glue(" 1")
+    } else {
+      str_add_list(out) <- stan_prior(
+        prior, class = "sderr", px = px, suffix = p,
+        comment = "SD of residuals", normalize = normalize
+      )
+    }
     str_add(out$tpar_def) <- glue(
       "  vector[N{resp}] err{p};  // actual residuals\n"
     )
@@ -1295,7 +1318,7 @@ stan_ac <- function(bterms, data, prior, threads, normalize, ...) {
       }
       str_add(out$data) <- glue(
         "  // number of lags per observation\n",
-        "  int<lower=0> J_lag{p}[N{resp}];\n"
+        "  array[N{resp}] int<lower=0> J_lag{p};\n"
       )
       str_add(out$model_def) <- glue(
         "  // matrix storing lagged residuals\n",
@@ -1340,24 +1363,42 @@ stan_ac <- function(bterms, data, prior, threads, normalize, ...) {
       )
     }
     if (acef_arma$p > 0) {
-      str_add_list(out) <- stan_prior(
-        prior, class = "ar", px = px, suffix = p,
-        coef = seq_along(acef_arma$p),
-        type = glue("vector[Kar{p}]"),
-        header_type = "vector",
-        comment = "autoregressive coefficients",
-        normalize = normalize
-      )
+      if (has_special_prior(prior, px, class = "ar")) {
+        if (acef_arma$cov) {
+          stop2("Cannot use shrinkage priors on 'ar' if cov = TRUE.")
+        }
+        str_add_list(out) <- stan_prior_non_centered(
+          class = "ar", suffix = p, suffix_K = "ar"
+        )
+      } else {
+        str_add_list(out) <- stan_prior(
+          prior, class = "ar", px = px, suffix = p,
+          coef = seq_along(acef_arma$p),
+          type = glue("vector[Kar{p}]"),
+          header_type = "vector",
+          comment = "autoregressive coefficients",
+          normalize = normalize
+        )
+      }
     }
     if (acef_arma$q > 0) {
-      str_add_list(out) <- stan_prior(
-        prior, class = "ma", px = px, suffix = p,
-        coef = seq_along(acef_arma$q),
-        type = glue("vector[Kma{p}]"),
-        header_type = "vector",
-        comment = "moving-average coefficients",
-        normalize = normalize
-      )
+      if (has_special_prior(prior, px, class = "ma")) {
+        if (acef_arma$cov) {
+          stop2("Cannot use shrinkage priors on 'ma' if cov = TRUE.")
+        }
+        str_add_list(out) <- stan_prior_non_centered(
+          class = "ma", suffix = p, suffix_K = "ma"
+        )
+      } else {
+        str_add_list(out) <- stan_prior(
+          prior, class = "ma", px = px, suffix = p,
+          coef = seq_along(acef_arma$q),
+          type = glue("vector[Kma{p}]"),
+          header_type = "vector",
+          comment = "moving-average coefficients",
+          normalize = normalize
+        )
+      }
     }
   }
 
@@ -1399,9 +1440,9 @@ stan_ac <- function(bterms, data, prior, threads, normalize, ...) {
     str_add(out$data) <- glue(
       "  // see the functions block for details\n",
       "  int<lower=1> N_tg{p};\n",
-      "  int<lower=1> begin_tg{p}[N_tg{p}];\n",
-      "  int<lower=1> end_tg{p}[N_tg{p}];\n",
-      "  int<lower=1> nobs_tg{p}[N_tg{p}];\n"
+      "  array[N_tg{p}] int<lower=1> begin_tg{p};\n",
+      "  array[N_tg{p}] int<lower=1> end_tg{p};\n",
+      "  array[N_tg{p}] int<lower=1> nobs_tg{p};\n"
     )
     str_add(out$pll_args) <- glue(
       ", int[] begin_tg{p}, int[] end_tg{p}, int[] nobs_tg{p}"
@@ -1415,7 +1456,7 @@ stan_ac <- function(bterms, data, prior, threads, normalize, ...) {
       # be represented directly via Cholesky factors due to potentially
       # different time subsets
       str_add(out$data) <- glue(
-        "  int<lower=0> Jtime_tg{p}[N_tg{p}, max(nobs_tg{p})];\n",
+        "  array[N_tg{p}, max(nobs_tg{p})] int<lower=0> Jtime_tg{p};\n",
         "  int n_unique_t{p};  // total number of unique time points\n",
         "  int n_unique_cortime{p};  // number of unique correlations\n"
       )
@@ -1514,15 +1555,23 @@ stan_ac <- function(bterms, data, prior, threads, normalize, ...) {
     str_add(out$data) <- glue(
       "  // data for the CAR structure\n",
       "  int<lower=1> Nloc{p};\n",
-      "  int<lower=1> Jloc{p}[N{resp}];\n",
+      "  array[N{resp}] int<lower=1> Jloc{p};\n",
       "  int<lower=0> Nedges{p};\n",
-      "  int<lower=1> edges1{p}[Nedges{p}];\n",
-      "  int<lower=1> edges2{p}[Nedges{p}];\n"
-    )
-    str_add_list(out) <- stan_prior(
-      prior, class = "sdcar", px = px, suffix = p,
-      comment = "SD of the CAR structure", normalize = normalize
+      "  array[Nedges{p}] int<lower=1> edges1{p};\n",
+      "  array[Nedges{p}] int<lower=1> edges2{p};\n"
     )
+    if (has_special_prior(prior, px, class = "sdcar")) {
+      str_add(out$tpar_def) <- glue(
+        "  real<lower=0> sdcar{p};  // SD of the CAR structure\n"
+      )
+      str_add(out$prior_global_scales) <- glue(" sdcar{p}")
+      str_add(out$prior_global_lengths) <- glue(" 1")
+    } else {
+      str_add_list(out) <- stan_prior(
+        prior, class = "sdcar", px = px, suffix = p,
+        comment = "SD of the CAR structure", normalize = normalize
+      )
+    }
     str_add(out$pll_args) <- glue(", vector rcar{p}, data int[] Jloc{p}")
     str_add(out$loopeta) <- glue(" + rcar{p}[Jloc{p}{n}]")
     if (acef_car$type %in% c("escar", "esicar")) {
@@ -1686,19 +1735,36 @@ stan_nl <- function(bterms, data, nlpars, threads, ...) {
       slice <- stan_slice(threads)
     }
     slice <- paste0(slice, " ")
-    str_add(out$data) <- "  // covariate vectors for non-linear functions\n"
+    str_add(out$data) <- "  // covariates for non-linear functions\n"
     for (i in seq_along(covars)) {
-      is_integer <- is.integer(data_cnl[[glue("C{p}_{i}")]])
+      cname <- glue("C{p}_{i}")
+      is_integer <- is.integer(data_cnl[[cname]])
+      is_matrix <- is.matrix(data_cnl[[cname]])
+      dim2 <- dim(data_cnl[[cname]])[2]
       if (is_integer) {
-        str_add(out$data) <- glue(
-          "  int C{p}_{i}[N{resp}];\n"
-        )
-        str_add(out$pll_args) <- glue(", data int[] C{p}_{i}")
+        if (is_matrix) {
+          str_add(out$data) <- glue(
+            "  array[N{resp}, {dim2}] int C{p}_{i};\n"
+          )
+          str_add(out$pll_args) <- glue(", data int[,] C{p}_{i}")
+        } else {
+          str_add(out$data) <- glue(
+            "  array[N{resp}] int C{p}_{i};\n"
+          )
+          str_add(out$pll_args) <- glue(", data int[] C{p}_{i}")
+        }
       } else {
-        str_add(out$data) <- glue(
-          "  vector[N{resp}] C{p}_{i};\n"
-        )
-        str_add(out$pll_args) <- glue(", data vector C{p}_{i}")
+        if (is_matrix) {
+          str_add(out$data) <- glue(
+            "  matrix[N{resp}, {dim2}] C{p}_{i};\n"
+          )
+          str_add(out$pll_args) <- glue(", data matrix C{p}_{i}")
+        } else {
+          str_add(out$data) <- glue(
+            "  vector[N{resp}] C{p}_{i};\n"
+          )
+          str_add(out$pll_args) <- glue(", data vector C{p}_{i}")
+        }
       }
       new_covars[i] <- glue(" C{p}_{i}{slice}")
     }
@@ -1766,7 +1832,7 @@ stan_Xme <- function(meef, prior, threads, normalize) {
       Nme <- glue("Nme_{i}")
       str_add(out$data) <- glue(
         "  int<lower=0> Nme_{i};  // number of latent values\n",
-        "  int<lower=1> Jme_{i}[N];  // group index per observation\n"
+        "  array[N] int<lower=1> Jme_{i};  // group index per observation\n"
       )
       str_add(out$pll_args) <- glue(", data int[] Jme_{i}")
     } else {
@@ -1862,8 +1928,8 @@ stan_Xme <- function(meef, prior, threads, normalize) {
 # @param primitive use Stan's GLM likelihood primitives?
 # @param ... currently unused
 # @return list of character strings containing Stan code
-stan_eta_combine <- function(out, bterms, ranef, threads, primitive, ...) {
-  stopifnot(is.list(out), is.btl(bterms))
+stan_eta_combine <- function(bterms, out, ranef, threads, primitive, ...) {
+  stopifnot(is.btl(bterms), is.list(out))
   if (primitive && !has_special_terms(bterms)) {
     # only overall effects and perhaps an intercept are present
     # which will be evaluated directly in the GLM primitive likelihood
@@ -2006,20 +2072,6 @@ stan_center_X <- function(x) {
     !fix_intercepts(x) && !is_sparse(x$fe) && !has_sum_to_zero_thres(x)
 }
 
-# indicate if the overall coefficients 'b' have a special prior
-stan_has_special_b_prior <- function(bterms, prior) {
-  special <- get_special_prior(prior, bterms)
-  !is.null(special$horseshoe) || !is.null(special$R2D2) ||
-    !is.null(special$lasso)
-}
-
-# indicate if the overall coefficients 'b' should be
-# assigned in the transformed parameters block
-stan_assign_b_tpar <- function(bterms, prior) {
-  special <- get_special_prior(prior, bterms)
-  !is.null(special$horseshoe) || !is.null(special$R2D2)
-}
-
 stan_dpar_comments <- function(dpar, family) {
   dpar_class <- dpar_class(dpar, family)
   out <- switch(dpar_class, "",
@@ -2060,7 +2112,7 @@ stan_dpar_transform <- function(bterms, prior, threads, normalize, ...) {
     len_mu <- glue("ncat{p}{str_if(is_logistic_normal, '-1')}")
     str_add(out$model_def) <- glue(
       "  // linear predictor matrix\n",
-      "  vector[{len_mu}] mu{p}[N{resp}];\n"
+      "  array[N{resp}] vector[{len_mu}] mu{p};\n"
     )
     mu_dpars <- make_stan_names(glue("mu{bterms$family$cats}"))
     mu_dpars <- glue("{mu_dpars}{p}[n]")
@@ -2149,7 +2201,7 @@ stan_dpar_transform <- function(bterms, prior, threads, normalize, ...) {
       # some of the sigmas are predicted
       str_add(out$model_def) <- glue(
         "  // sigma parameter matrix\n",
-        "  vector[{ncatm1}] sigma{p}[N{resp}];\n"
+        "  array[N{resp}] vector[{ncatm1}] sigma{p};\n"
       )
       str_add(out$model_comp_catjoin) <- glue(
         "  for (n in 1:N{resp}) {{\n",
