@@ -29,12 +29,12 @@ stan_response <- function(bterms, data, normalize) {
   if (has_multicol(family)) {
     if (rtype == "real") {
       str_add(out$data) <- glue(
-        "  vector[ncat{resp}] Y{resp}[N{resp}];  // response array\n"
+        "  array[N{resp}] vector[ncat{resp}] Y{resp};  // response array\n"
       )
       str_add(out$pll_args) <- glue(", data vector[] Y{resp}")
     } else if (rtype == "int") {
       str_add(out$data) <- glue(
-        "  int Y{resp}[N{resp}, ncat{resp}];  // response array\n"
+        "  array[N{resp}, ncat{resp}] int Y{resp};  // response array\n"
       )
       str_add(out$pll_args) <- glue(", data int[,] Y{resp}")
     }
@@ -47,7 +47,7 @@ stan_response <- function(bterms, data, normalize) {
       str_add(out$pll_args) <- glue(", data vector Y{resp}")
     } else if (rtype == "int") {
       str_add(out$data) <- glue(
-        "  int Y{resp}[N{resp}];  // response variable\n"
+        "  array[N{resp}] int Y{resp};  // response variable\n"
       )
       str_add(out$pll_args) <- glue(", data int[] Y{resp}")
     }
@@ -59,7 +59,7 @@ stan_response <- function(bterms, data, normalize) {
   }
   if (has_trials(family) || is.formula(bterms$adforms$trials)) {
     str_add(out$data) <- glue(
-      "  int trials{resp}[N{resp}];  // number of trials\n"
+      "  array[N{resp}] int trials{resp};  // number of trials\n"
     )
     str_add(out$pll_args) <- glue(", data int[] trials{resp}")
   }
@@ -74,15 +74,15 @@ stan_response <- function(bterms, data, normalize) {
     if (any(nzchar(groups))) {
       str_add(out$data) <- glue(
         "  int<lower=1> ngrthres{resp};  // number of threshold groups\n",
-        "  int<lower=1> nthres{resp}[ngrthres{resp}];  // number of thresholds\n",
-        "  int<lower=1> Jthres{resp}[N{resp}, 2];  // threshold indices\n"
+        "  array[ngrthres{resp}] int<lower=1> nthres{resp};  // number of thresholds\n",
+        "  array[N{resp}, 2] int<lower=1> Jthres{resp};  // threshold indices\n"
       )
       str_add(out$tdata_def) <- glue(
         "  int<lower=1> nmthres{resp} = sum(nthres{resp});",
         "  // total number of thresholds\n",
-        "  int<lower=1> Kthres_start{resp}[ngrthres{resp}];",
+        "  array[ngrthres{resp}] int<lower=1> Kthres_start{resp};",
         "  // start index per threshold group\n",
-        "  int<lower=1> Kthres_end{resp}[ngrthres{resp}];",
+        "  array[ngrthres{resp}] int<lower=1> Kthres_end{resp};",
         "  // end index per threshold group\n"
       )
       str_add(out$tdata_comp) <- glue(
@@ -116,7 +116,7 @@ stan_response <- function(bterms, data, normalize) {
   }
   if (is.formula(bterms$adforms$dec)) {
     str_add(out$data) <- glue(
-      "  int<lower=0,upper=1> dec{resp}[N{resp}];  // decisions\n"
+      "  array[N{resp}] int<lower=0,upper=1> dec{resp};  // decisions\n"
     )
     str_add(out$pll_args) <- glue(", data int[] dec{resp}")
   }
@@ -135,7 +135,7 @@ stan_response <- function(bterms, data, normalize) {
   }
   if (is.formula(bterms$adforms$cens)) {
     str_add(out$data) <- glue(
-      "  int<lower=-1,upper=2> cens{resp}[N{resp}];  // indicates censoring\n"
+      "  array[N{resp}] int<lower=-1,upper=2> cens{resp};  // indicates censoring\n"
     )
     str_add(out$pll_args) <- glue(", data int[] cens{resp}")
     y2_expr <- get_ad_expr(bterms, "cens", "y2")
@@ -143,7 +143,7 @@ stan_response <- function(bterms, data, normalize) {
       # interval censoring is required
       if (rtype == "int") {
         str_add(out$data) <- glue(
-          "  int rcens{resp}[N{resp}];"
+          "  array[N{resp}] int rcens{resp};"
         )
         str_add(out$pll_args) <- glue(", data int[] rcens{resp}")
       } else {
@@ -158,13 +158,13 @@ stan_response <- function(bterms, data, normalize) {
   bounds <- trunc_bounds(bterms, data = data)
   if (any(bounds$lb > -Inf)) {
     str_add(out$data) <- glue(
-      "  {rtype} lb{resp}[N{resp}];  // lower truncation bounds;\n"
+      "  array[N{resp}] {rtype} lb{resp};  // lower truncation bounds;\n"
     )
     str_add(out$pll_args) <- glue(", data {rtype}[] lb{resp}")
   }
   if (any(bounds$ub < Inf)) {
     str_add(out$data) <- glue(
-      "  {rtype} ub{resp}[N{resp}];  // upper truncation bounds\n"
+      "  array[N{resp}] {rtype} ub{resp};  // upper truncation bounds\n"
     )
     str_add(out$pll_args) <- glue(", data {rtype}[] ub{resp}")
   }
@@ -176,7 +176,7 @@ stan_response <- function(bterms, data, normalize) {
       # response is modeled without measurement error
       str_add(out$data) <- glue(
         "  int<lower=0> Nmi{resp};  // number of missings\n",
-        "  int<lower=1> Jmi{resp}[Nmi{resp}];  // positions of missings\n"
+        "  array[Nmi{resp}] int<lower=1> Jmi{resp};  // positions of missings\n"
       )
       str_add(out$par) <- glue(
         "  vector{Ybounds}[Nmi{resp}] Ymi{resp};  // estimated missings\n"
@@ -195,7 +195,7 @@ stan_response <- function(bterms, data, normalize) {
         "  vector<lower=0>[N{resp}] noise{resp};\n",
         "  // information about non-missings\n",
         "  int<lower=0> Nme{resp};\n",
-        "  int<lower=1> Jme{resp}[Nme{resp}];\n"
+        "  array[Nme{resp}] int<lower=1> Jme{resp};\n"
       )
       str_add(out$par) <- glue(
         "  vector{Ybounds}[N{resp}] Yl{resp};  // latent variable\n"
@@ -213,7 +213,7 @@ stan_response <- function(bterms, data, normalize) {
     k <- length(vreal$vars)
     str_add(out$data) <- cglue(
       "  // data for custom real vectors\n",
-      "  real vreal{seq_len(k)}{resp}[N{resp}];\n"
+      "  array[N{resp}] real vreal{seq_len(k)}{resp};\n"
     )
     str_add(out$pll_args) <- cglue(", data real[] vreal{seq_len(k)}{resp}")
   }
@@ -223,7 +223,7 @@ stan_response <- function(bterms, data, normalize) {
     k <- length(vint$vars)
     str_add(out$data) <- cglue(
       "  // data for custom integer vectors\n",
-      "  int vint{seq_len(k)}{resp}[N{resp}];\n"
+      "  array[N{resp}] int vint{seq_len(k)}{resp};\n"
     )
     str_add(out$pll_args) <- cglue(", data int[] vint{seq_len(k)}{resp}")
   }
@@ -493,7 +493,7 @@ stan_mixture <- function(bterms, data, prior, threads, normalize, ...) {
 # @return a character string
 stan_ordinal_lpmf <- function(family, link) {
   stopifnot(is.character(family), is.character(link))
-  inv_link <- stan_inv_link(link)
+  inv_link <- stan_inv_link(link, vectorize = FALSE)
   th <- function(k) {
     # helper function generating stan code inside inv_link(.)
     if (family %in% c("cumulative", "sratio")) {
@@ -663,3 +663,122 @@ stan_ordinal_lpmf <- function(family, link) {
   }
   out
 }
+
+# log probability density for hurdle ordinal models
+# @return a character string
+stan_hurdle_ordinal_lpmf <- function(family, link) {
+  stopifnot(is.character(family), is.character(link))
+  # TODO: generalize to non-cumulative families?
+  stopifnot(family == "hurdle_cumulative")
+  inv_link <- stan_inv_link(link, vectorize = FALSE)
+  th <- function(k) {
+    out <- glue("thres[{k}] - mu")
+    glue("disc * ({out})")
+  }
+  out <- glue(
+    "  /* {family}-{link} log-PDF for a single response\n",
+    "   * Args:\n",
+    "   *   y: response category\n",
+    "   *   mu: latent mean parameter\n",
+    "   *   hu: hurdle probability\n",
+    "   *   disc: discrimination parameter\n",
+    "   *   thres: ordinal thresholds\n",
+    "   * Returns:\n",
+    "   *   a scalar to be added to the log posterior\n",
+    "   */\n",
+    "   real {family}_{link}_lpmf(int y, real mu, real hu, real disc, vector thres) {{\n",
+    "\n"
+  )
+  # define the function body
+  if (inv_link == "inv_logit") {
+    str_add(out) <- glue(
+      "     int nthres = num_elements(thres);\n",
+      "     if (y == 0) {{\n",
+      "       return bernoulli_lpmf(1 | hu);\n",
+      "     }} else if (y == 1) {{\n",
+      "       return log_inv_logit({th(1)}) +\n",
+      "                bernoulli_lpmf(0 | hu);\n",
+      "     }} else if (y == nthres + 2) {{\n",
+      "       return log1m_inv_logit({th('nthres')}) +\n",
+      "                bernoulli_lpmf(0 | hu);\n",
+      "     }} else {{\n",
+      # TODO: replace with log_inv_logit_diff once rstan >= 2.25
+      "       return log_diff_exp(\n",
+      "         log_inv_logit({th('y')}), \n",
+      "         log_inv_logit({th('y - 1')})\n",
+      "       ) + bernoulli_lpmf(0 | hu) ;\n",
+      "     }}\n",
+      "   }}\n"
+    )
+  } else {
+    str_add(out) <- glue(
+      "     int nthres = num_elements(thres);\n",
+      "     real p;\n",
+      "     if (y == 0){{\n",
+      "       p = hu;\n",
+      "     }} else if (y == 1) {{\n",
+      "       p = {inv_link}({th(1)}) * (1 - hu);\n",
+      "     }} else if (y == nthres + 1) {{\n",
+      "       p = (1 - {inv_link}({th('nthres')})) * (1 - hu);\n",
+      "     }} else {{\n",
+      "       p = ({inv_link}({th('y')}) -\n",
+      "           {inv_link}({th('y - 1')})) * (1 - hu);\n",
+      "     }}\n",
+      "     return log(p);\n",
+      "   }}\n"
+    )
+  }
+
+  # lpmf function for multiple merged thresholds
+  str_add(out) <- glue(
+    "  /* {family}-{link} log-PDF for a single response and merged thresholds\n",
+    "   * Args:\n",
+    "   *   y: response category\n",
+    "   *   mu: latent mean parameter\n",
+    "   *   hu: hurdle probability\n",
+    "   *   disc: discrimination parameter\n",
+    "   *   thres: vector of merged ordinal thresholds\n",
+    "   *   j: start and end index for the applid threshold within 'thres'\n",
+    "   * Returns:\n",
+    "   *   a scalar to be added to the log posterior\n",
+    "   */\n",
+    "   real {family}_{link}_merged_lpmf(",
+    "int y, real mu, real hu, real disc, vector thres, int[] j) {{\n",
+    "     return {family}_{link}_lpmf(y | mu, hu, disc, thres[j[1]:j[2]]);\n",
+    "   }}\n"
+  )
+
+  if (link == "logit") {
+    # use the more efficient ordered_logistic function when disc == 1
+    str_add(out) <- glue(
+      "\n",
+      " // Use more efficient ordered_logistic function with disc == 1\n",
+      "   real hurdle_cumulative_ordered_logistic_lpmf(int y, real mu, real hu, real disc, vector thres) {{\n",
+      "     if (y == 0) {{\n",
+      "       return bernoulli_lpmf(1 | hu);\n",
+      "     }} else {{\n",
+      "       return ordered_logistic_lpmf(y | mu, thres) +\n",
+      "                bernoulli_lpmf(0 | hu);\n",
+      "     }}\n",
+      "   }}\n"
+    )
+    str_add(out) <- glue(
+      "  /* use ordered-logistic log-PDF for a single response and merged thresholds\n",
+      "   * Args:\n",
+      "   *   y: response category\n",
+      "   *   mu: latent mean parameter\n",
+      "   *   hu: hurdle probability\n",
+      "   *   thres: vector of merged ordinal thresholds\n",
+      "   *   j: start and end index for the applid threshold within 'thres'\n",
+      "   * Returns:\n",
+      "   *   a scalar to be added to the log posterior\n",
+      "   */\n",
+      "   real hurdle_cumulative_ordered_logistic_merged_lpmf(",
+      "int y, real mu, real hu, real disc, vector thres, int[] j) {{\n",
+      "     return hurdle_cumulative_ordered_logistic_lpmf(y | mu, hu, disc, thres[j[1]:j[2]]);\n",
+      "   }}\n"
+    )
+  }
+  out
+}
+
