@@ -667,6 +667,12 @@ test_that("pp_check has reasonable outputs", {
 
 test_that("posterior_epred has reasonable outputs", {
   expect_equal(dim(posterior_epred(fit1)), c(ndraws(fit1), nobs(fit1)))
+
+  # test that point_estimate produces identical draws
+  pe <- posterior_epred(fit1, point_estimate = "median",
+                        ndraws_point_estimate = 2)
+  expect_equal(nrow(pe), 2)
+  expect_true(all(pe[1, ] == pe[2, ]))
 })
 
 test_that("pp_mixture has reasonable outputs", {
@@ -743,7 +749,7 @@ test_that("prior_draws has reasonable outputs", {
   prs1 <- prior_draws(fit1)
   prior_names <- c(
     "Intercept", "b", paste0("simo_moExp1[", 1:4, "]"), "bsp",
-    "bs", "sds_sAge_1", "b_sigma", "Intercept_sigma", "nu",
+    "bs", "sds_sAge", "b_sigma", "Intercept_sigma", "nu",
     "sd_visit", "cor_visit"
   )
   expect_equal(colnames(prs1), prior_names)
@@ -809,9 +815,9 @@ test_that("stancode has reasonable outputs", {
 
 test_that("standata has reasonable outputs", {
   expect_equal(sort(names(standata(fit1))),
-    sort(c("N", "Y",  "Kar", "Kma", "J_lag", "K", "X", "Ksp", "Imo",
+    sort(c("N", "Y", "Kar", "Kma", "J_lag", "K", "Kc", "X", "Ksp", "Imo",
            "Xmo_1", "Jmo", "con_simo_1", "Z_1_1", "Z_1_2", "nb_1",
-           "knots_1", "Zs_1_1", "Ks", "Xs", "offsets", "K_sigma",
+           "knots_1", "Zs_1_1", "Ks", "Xs", "offsets", "K_sigma", "Kc_sigma",
            "X_sigma", "J_1", "N_1", "M_1", "NC_1", "prior_only"))
   )
   expect_equal(sort(names(standata(fit2))),
@@ -980,10 +986,10 @@ test_that("waic has reasonable outputs", {
 })
 
 test_that("diagnostic convenience functions have reasonable outputs", {
-  expect_true(is(log_posterior(fit1), "data.frame"))
-  expect_true(is(nuts_params(fit1), "data.frame"))
-  expect_true(is(rhat(fit1), "numeric"))
-  expect_true(is(neff_ratio(fit1), "numeric"))
+  expect_true(is.data.frame(log_posterior(fit1)))
+  expect_true(is.data.frame(nuts_params(fit1)))
+  expect_true(is.numeric(rhat(fit1)))
+  expect_true(is.numeric(SW(neff_ratio(fit1))))
 })
 
 test_that("contrasts of grouping factors are not stored #214", {
